package bou.amine.apps.readerforselfossv2.android.viewmodel

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import bou.amine.apps.readerforselfossv2.android.R
import bou.amine.apps.readerforselfossv2.model.SelfossModel
import bou.amine.apps.readerforselfossv2.repository.Repository
import bou.amine.apps.readerforselfossv2.utils.ItemType
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.MutableSharedFlow
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asSharedFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.launch

class AppViewModel(private val repository: Repository) : ViewModel() {
    private val _networkAvailableProvider = MutableSharedFlow<Boolean>()
    val networkAvailableProvider = _networkAvailableProvider.asSharedFlow()
    private val _refreshingIndicatorProvider = MutableSharedFlow<Boolean>()
    val refreshingIndicatorProvider = _refreshingIndicatorProvider.asSharedFlow()
    private val _toastMessageProvider = MutableSharedFlow<Int>()
    val toastMessageProvider = _toastMessageProvider.asSharedFlow()
    private var wasConnected = true

    private val _items = MutableStateFlow(ArrayList<SelfossModel.Item>())
    val items = _items.asStateFlow()

    init {
        viewModelScope.launch {
            repository.isConnectionAvailable.collect { isConnected ->
                if (repository.connectionMonitored) {
                    if (isConnected && !wasConnected && repository.connectionMonitored) {
                        _networkAvailableProvider.emit(true)
                        wasConnected = true
                    } else if (!isConnected && wasConnected && repository.connectionMonitored){
                        _networkAvailableProvider.emit(false)
                        wasConnected = false
                    }
                }
            }
        }
    }

    fun updateRemote() {
        CoroutineScope(Dispatchers.IO).launch {
            _toastMessageProvider.emit(R.string.refresh_in_progress)
            val updatedRemote = repository.updateRemote()
            if (updatedRemote) {
                _toastMessageProvider.emit(R.string.refresh_success_response)
            } else {
                _toastMessageProvider.emit(R.string.refresh_failer_message)
            }
        }
    }

    fun getItems(appendResults: Boolean, itemType: ItemType) {
        // TODO: Find a way to use Dispatchers.IO without creating conflicts
        CoroutineScope(Dispatchers.Main).launch {
            _refreshingIndicatorProvider.emit(true)
            repository.displayedItems = itemType
            val items = if (appendResults) {
                repository.getOlderItems()
            } else {
                repository.getNewerItems()
            }
            _items.emit(items)
            _refreshingIndicatorProvider.emit(false)
        }
    }

    fun markAllAsRead() {
        CoroutineScope(Dispatchers.IO).launch {
            _refreshingIndicatorProvider.emit(true)
            val success = repository.markAllAsRead(items.value)
            if (success) {
                _toastMessageProvider.emit(R.string.all_posts_read)
            } else {
                _toastMessageProvider.emit(R.string.all_posts_not_read)
            }
            _refreshingIndicatorProvider.emit(false)
        }
    }
}